@@ -1,6 +1,6 @@
-from typing import Dict, Iterable, List, Optional, Set
+from typing import Dict, Iterable, List, Optional, Set, Union
 
-from mypy.types import SyntheticTypeVisitor
+from mypy.types import TypeVisitor
 import mypy.types as types
 from mypy.util import split_module_names
 
@@ -15,18 +15,26 @@ def extract_module_names(type_name: Optional[str]) -> List[str]:
         return []
 
 
-class TypeIndirectionVisitor(SyntheticTypeVisitor[Set[str]]):
+class TypeIndirectionVisitor(TypeVisitor[Set[str]]):
     """Returns all module references within a particular type."""
 
     def __init__(self) -> None:
         self.cache = {}  # type: Dict[types.Type, Set[str]]
+        self.seen_aliases = set()  # type: Set[types.TypeAliasType]
 
     def find_modules(self, typs: Iterable[types.Type]) -> Set[str]:
-        return self._visit(*typs)
+        self.seen_aliases.clear()
+        return self._visit(typs)
 
-    def _visit(self, *typs: types.Type) -> Set[str]:
+    def _visit(self, typ_or_typs: Union[types.Type, Iterable[types.Type]]) -> Set[str]:
+        typs = [typ_or_typs] if isinstance(typ_or_typs, types.Type) else typ_or_typs
         output = set()  # type: Set[str]
         for typ in typs:
+            if isinstance(typ, types.TypeAliasType):
+                # Avoid infinite recursion for recursive type aliases.
+                if typ in self.seen_aliases:
+                    continue
+                self.seen_aliases.add(typ)
             if typ in self.cache:
                 modules = self.cache[typ]
             else:
@@ -36,18 +44,12 @@ class TypeIndirectionVisitor(SyntheticTypeVisitor[Set[str]]):
         return output
 
     def visit_unbound_type(self, t: types.UnboundType) -> Set[str]:
-        return self._visit(*t.args)
-
-    def visit_type_list(self, t: types.TypeList) -> Set[str]:
-        return self._visit(*t.items)
-
-    def visit_callable_argument(self, t: types.CallableArgument) -> Set[str]:
-        return self._visit(t.typ)
+        return self._visit(t.args)
 
     def visit_any(self, t: types.AnyType) -> Set[str]:
         return set()
 
-    def visit_none_type(self, t: types.NoneTyp) -> Set[str]:
+    def visit_none_type(self, t: types.NoneType) -> Set[str]:
         return set()
 
     def visit_uninhabited_type(self, t: types.UninhabitedType) -> Set[str]:
@@ -60,11 +62,11 @@ class TypeIndirectionVisitor(SyntheticTypeVisitor[Set[str]]):
         return set()
 
     def visit_type_var(self, t: types.TypeVarType) -> Set[str]:
-        return self._visit(*t.values) | self._visit(t.upper_bound)
+        return self._visit(t.values) | self._visit(t.upper_bound)
 
     def visit_instance(self, t: types.Instance) -> Set[str]:
-        out = self._visit(*t.args)
-        if t.type is not None:
+        out = self._visit(t.args)
+        if t.type:
             # Uses of a class depend on everything in the MRO,
             # as changes to classes in the MRO can add types to methods,
             # change property types, change the MRO itself, etc.
@@ -75,37 +77,31 @@ class TypeIndirectionVisitor(SyntheticTypeVisitor[Set[str]]):
         return out
 
     def visit_callable_type(self, t: types.CallableType) -> Set[str]:
-        out = self._visit(*t.arg_types) | self._visit(t.ret_type)
+        out = self._visit(t.arg_types) | self._visit(t.ret_type)
         if t.definition is not None:
-            out.update(extract_module_names(t.definition.fullname()))
+            out.update(extract_module_names(t.definition.fullname))
         return out
 
     def visit_overloaded(self, t: types.Overloaded) -> Set[str]:
-        return self._visit(*t.items()) | self._visit(t.fallback)
+        return self._visit(t.items()) | self._visit(t.fallback)
 
     def visit_tuple_type(self, t: types.TupleType) -> Set[str]:
-        return self._visit(*t.items) | self._visit(t.fallback)
+        return self._visit(t.items) | self._visit(t.partial_fallback)
 
     def visit_typeddict_type(self, t: types.TypedDictType) -> Set[str]:
-        return self._visit(*t.items.values()) | self._visit(t.fallback)
+        return self._visit(t.items.values()) | self._visit(t.fallback)
 
-    def visit_star_type(self, t: types.StarType) -> Set[str]:
-        return set()
+    def visit_literal_type(self, t: types.LiteralType) -> Set[str]:
+        return self._visit(t.fallback)
 
     def visit_union_type(self, t: types.UnionType) -> Set[str]:
-        return self._visit(*t.items)
+        return self._visit(t.items)
 
     def visit_partial_type(self, t: types.PartialType) -> Set[str]:
         return set()
 
-    def visit_ellipsis_type(self, t: types.EllipsisType) -> Set[str]:
-        return set()
-
     def visit_type_type(self, t: types.TypeType) -> Set[str]:
         return self._visit(t.item)
 
-    def visit_forwardref_type(self, t: types.ForwardRef) -> Set[str]:
-        if t.resolved:
-            return self._visit(t.resolved)
-        else:
-            return set()
+    def visit_type_alias_type(self, t: types.TypeAliasType) -> Set[str]:
+        return self._visit(types.get_proper_type(t))
