@@ -1,14 +1,17 @@
-from collections import OrderedDict
+from mypy.ordered_dict import OrderedDict
 import re
 import pprint
 import sys
 
-from typing_extensions import Final
+from typing_extensions import Final, TYPE_CHECKING
 from typing import Dict, List, Mapping, Optional, Pattern, Set, Tuple, Callable, Any
 
 from mypy import defaults
 from mypy.util import get_class_descriptors, replace_object_state
 
+if TYPE_CHECKING:
+    from mypy.errors import ErrorCode
+
 
 class BuildType:
     STANDARD = 0  # type: Final[int]
@@ -18,9 +21,8 @@ class BuildType:
 
 PER_MODULE_OPTIONS = {
     # Please keep this list sorted
-    "allow_untyped_globals",
     "allow_redefinition",
-    "strict_equality",
+    "allow_untyped_globals",
     "always_false",
     "always_true",
     "check_untyped_defs",
@@ -39,11 +41,12 @@ PER_MODULE_OPTIONS = {
     "follow_imports_for_stubs",
     "ignore_errors",
     "ignore_missing_imports",
+    "implicit_reexport",
     "local_partial_types",
     "mypyc",
     "no_implicit_optional",
-    "implicit_reexport",
     "show_none_errors",
+    "strict_equality",
     "strict_optional",
     "strict_optional_whitelist",
     "warn_no_return",
@@ -77,13 +80,25 @@ class Options:
         self.report_dirs = {}  # type: Dict[str, str]
         # Show errors in PEP 561 packages/site-packages modules
         self.no_silence_site_packages = False
+        self.no_site_packages = False
         self.ignore_missing_imports = False
         self.follow_imports = 'normal'  # normal|silent|skip|error
         # Whether to respect the follow_imports setting even for stub files.
         # Intended to be used for disabling specific stubs.
         self.follow_imports_for_stubs = False
         # PEP 420 namespace packages
+        # This allows definitions of packages without __init__.py and allows packages to span
+        # multiple directories. This flag affects both import discovery and the association of
+        # input files/modules/packages to the relevant file and fully qualified module name.
         self.namespace_packages = False
+        # Use current directory and MYPYPATH to determine fully qualified module names of files
+        # passed by automatically considering their subdirectories as packages. This is only
+        # relevant if namespace packages are enabled, since otherwise examining __init__.py's is
+        # sufficient to determine module names for files. As a possible alternative, add a single
+        # top-level __init__.py to your packages.
+        self.explicit_package_bases = False
+        # File names, directory names or subpaths to avoid checking
+        self.exclude = ""  # type: str
 
         # disallow_any options
         self.disallow_any_generics = False
@@ -176,6 +191,14 @@ class Options:
         # Variable names considered False
         self.always_false = []  # type: List[str]
 
+        # Error codes to disable
+        self.disable_error_code = []  # type: List[str]
+        self.disabled_error_codes = set()  # type: Set[ErrorCode]
+
+        # Error codes to enable
+        self.enable_error_code = []  # type: List[str]
+        self.enabled_error_codes = set()  # type: Set[ErrorCode]
+
         # Use script name instead of __main__
         self.scripts_are_modules = False
 
@@ -217,6 +240,9 @@ class Options:
         # mypy. (Like mypyc.)
         self.preserve_asts = False
 
+        # PEP 612 support is a work in progress, hide it from users
+        self.wip_pep_612 = False
+
         # Paths of user plugins
         self.plugins = []  # type: List[str]
 
