@@ -1,16 +1,20 @@
 from functools import partial
-from typing import Callable, Optional
+from typing import Callable, Optional, List
 
 from mypy import message_registry
-from mypy.nodes import StrExpr, IntExpr, DictExpr, UnaryExpr
+from mypy.nodes import Expression, StrExpr, IntExpr, DictExpr, UnaryExpr
 from mypy.plugin import (
-    Plugin, FunctionContext, MethodContext, MethodSigContext, AttributeContext, ClassDefContext
+    Plugin, FunctionContext, MethodContext, MethodSigContext, AttributeContext, ClassDefContext,
+    CheckerPluginInterface,
 )
-from mypy.plugins.common import try_getting_str_literal
+from mypy.plugins.common import try_getting_str_literals
 from mypy.types import (
-    Type, Instance, AnyType, TypeOfAny, CallableType, NoneType, UnionType, TypedDictType,
-    TypeVarType, TPDICT_FB_NAMES
+    Type, Instance, AnyType, TypeOfAny, CallableType, NoneType, TypedDictType,
+    TypeVarDef, TypeVarType, TPDICT_FB_NAMES, get_proper_type, LiteralType
 )
+from mypy.subtypes import is_subtype
+from mypy.typeops import make_simplified_union
+from mypy.checkexpr import is_literal_type_like
 
 
 class DefaultPlugin(Plugin):
@@ -54,6 +58,8 @@ class DefaultPlugin(Plugin):
             return typed_dict_get_callback
         elif fullname == 'builtins.int.__pow__':
             return int_pow_callback
+        elif fullname == 'builtins.int.__neg__':
+            return int_neg_callback
         elif fullname in set(n + '.setdefault' for n in TPDICT_FB_NAMES):
             return typed_dict_setdefault_callback
         elif fullname in set(n + '.pop' for n in TPDICT_FB_NAMES):
@@ -64,6 +70,8 @@ class DefaultPlugin(Plugin):
             return ctypes.array_getitem_callback
         elif fullname == 'ctypes.Array.__iter__':
             return ctypes.array_iter_callback
+        elif fullname == 'pathlib.Path.open':
+            return path_open_callback
         return None
 
     def get_attribute_hook(self, fullname: str
@@ -91,7 +99,18 @@ class DefaultPlugin(Plugin):
         elif fullname in attrs.attr_dataclass_makers:
             return partial(
                 attrs.attr_class_maker_callback,
-                auto_attribs_default=True
+                auto_attribs_default=True,
+            )
+        elif fullname in attrs.attr_frozen_makers:
+            return partial(
+                attrs.attr_class_maker_callback,
+                auto_attribs_default=None,
+                frozen_default=True,
+            )
+        elif fullname in attrs.attr_define_makers:
+            return partial(
+                attrs.attr_class_maker_callback,
+                auto_attribs_default=None,
             )
         elif fullname in dataclasses.dataclass_makers:
             return dataclasses.dataclass_class_maker_callback
@@ -99,35 +118,68 @@ class DefaultPlugin(Plugin):
 
 
 def open_callback(ctx: FunctionContext) -> Type:
-    """Infer a better return type for 'open'.
+    """Infer a better return type for 'open'."""
+    return _analyze_open_signature(
+        arg_types=ctx.arg_types,
+        args=ctx.args,
+        mode_arg_index=1,
+        default_return_type=ctx.default_return_type,
+        api=ctx.api,
+    )
 
-    Infer TextIO or BinaryIO as the return value if the mode argument is not
-    given or is a literal.
+
+def path_open_callback(ctx: MethodContext) -> Type:
+    """Infer a better return type for 'pathlib.Path.open'."""
+    return _analyze_open_signature(
+        arg_types=ctx.arg_types,
+        args=ctx.args,
+        mode_arg_index=0,
+        default_return_type=ctx.default_return_type,
+        api=ctx.api,
+    )
+
+
+def _analyze_open_signature(arg_types: List[List[Type]],
+                            args: List[List[Expression]],
+                            mode_arg_index: int,
+                            default_return_type: Type,
+                            api: CheckerPluginInterface,
+                            ) -> Type:
+    """A helper for analyzing any function that has approximately
+    the same signature as the builtin 'open(...)' function.
+
+    Currently, the only thing the caller can customize is the index
+    of the 'mode' argument. If the mode argument is omitted or is a
+    string literal, we refine the return type to either 'TextIO' or
+    'BinaryIO' as appropriate.
     """
     mode = None
-    if not ctx.arg_types or len(ctx.arg_types[1]) != 1:
+    if not arg_types or len(arg_types[mode_arg_index]) != 1:
         mode = 'r'
-    elif isinstance(ctx.args[1][0], StrExpr):
-        mode = ctx.args[1][0].value
+    else:
+        mode_expr = args[mode_arg_index][0]
+        if isinstance(mode_expr, StrExpr):
+            mode = mode_expr.value
     if mode is not None:
-        assert isinstance(ctx.default_return_type, Instance)
+        assert isinstance(default_return_type, Instance)  # type: ignore
         if 'b' in mode:
-            return ctx.api.named_generic_type('typing.BinaryIO', [])
+            return api.named_generic_type('typing.BinaryIO', [])
         else:
-            return ctx.api.named_generic_type('typing.TextIO', [])
-    return ctx.default_return_type
+            return api.named_generic_type('typing.TextIO', [])
+    return default_return_type
 
 
 def contextmanager_callback(ctx: FunctionContext) -> Type:
     """Infer a better return type for 'contextlib.contextmanager'."""
     # Be defensive, just in case.
     if ctx.arg_types and len(ctx.arg_types[0]) == 1:
-        arg_type = ctx.arg_types[0][0]
+        arg_type = get_proper_type(ctx.arg_types[0][0])
+        default_return = get_proper_type(ctx.default_return_type)
         if (isinstance(arg_type, CallableType)
-                and isinstance(ctx.default_return_type, CallableType)):
+                and isinstance(default_return, CallableType)):
             # The stub signature doesn't preserve information about arguments so
             # add them back here.
-            return ctx.default_return_type.copy_modified(
+            return default_return.copy_modified(
                 arg_types=arg_type.arg_types,
                 arg_kinds=arg_type.arg_kinds,
                 arg_names=arg_type.arg_names,
@@ -151,7 +203,7 @@ def typed_dict_get_signature_callback(ctx: MethodSigContext) -> CallableType:
             and len(signature.variables) == 1
             and len(ctx.args[1]) == 1):
         key = ctx.args[0][0].value
-        value_type = ctx.type.items.get(key)
+        value_type = get_proper_type(ctx.type.items.get(key))
         ret_type = signature.ret_type
         if value_type:
             default_arg = ctx.args[1][0]
@@ -163,10 +215,11 @@ def typed_dict_get_signature_callback(ctx: MethodSigContext) -> CallableType:
             # Tweak the signature to include the value type as context. It's
             # only needed for type inference since there's a union with a type
             # variable that accepts everything.
+            assert isinstance(signature.variables[0], TypeVarDef)
             tv = TypeVarType(signature.variables[0])
             return signature.copy_modified(
                 arg_types=[signature.arg_types[0],
-                           UnionType.make_simplified_union([value_type, tv])],
+                           make_simplified_union([value_type, tv])],
                 ret_type=ret_type)
     return signature
 
@@ -176,26 +229,34 @@ def typed_dict_get_callback(ctx: MethodContext) -> Type:
     if (isinstance(ctx.type, TypedDictType)
             and len(ctx.arg_types) >= 1
             and len(ctx.arg_types[0]) == 1):
-        key = try_getting_str_literal(ctx.args[0][0], ctx.arg_types[0][0])
-        if key is None:
+        keys = try_getting_str_literals(ctx.args[0][0], ctx.arg_types[0][0])
+        if keys is None:
             return ctx.default_return_type
 
-        value_type = ctx.type.items.get(key)
-        if value_type:
+        output_types = []  # type: List[Type]
+        for key in keys:
+            value_type = get_proper_type(ctx.type.items.get(key))
+            if value_type is None:
+                ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
+                return AnyType(TypeOfAny.from_error)
+
             if len(ctx.arg_types) == 1:
-                return UnionType.make_simplified_union([value_type, NoneType()])
+                output_types.append(value_type)
             elif (len(ctx.arg_types) == 2 and len(ctx.arg_types[1]) == 1
                   and len(ctx.args[1]) == 1):
                 default_arg = ctx.args[1][0]
                 if (isinstance(default_arg, DictExpr) and len(default_arg.items) == 0
                         and isinstance(value_type, TypedDictType)):
                     # Special case '{}' as the default for a typed dict type.
-                    return value_type.copy_modified(required_keys=set())
+                    output_types.append(value_type.copy_modified(required_keys=set()))
                 else:
-                    return UnionType.make_simplified_union([value_type, ctx.arg_types[1][0]])
-        else:
-            ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
-            return AnyType(TypeOfAny.from_error)
+                    output_types.append(value_type)
+                    output_types.append(ctx.arg_types[1][0])
+
+        if len(ctx.arg_types) == 1:
+            output_types.append(NoneType())
+
+        return make_simplified_union(output_types)
     return ctx.default_return_type
 
 
@@ -220,8 +281,9 @@ def typed_dict_pop_signature_callback(ctx: MethodSigContext) -> CallableType:
             # Tweak the signature to include the value type as context. It's
             # only needed for type inference since there's a union with a type
             # variable that accepts everything.
+            assert isinstance(signature.variables[0], TypeVarDef)
             tv = TypeVarType(signature.variables[0])
-            typ = UnionType.make_simplified_union([value_type, tv])
+            typ = make_simplified_union([value_type, tv])
             return signature.copy_modified(
                 arg_types=[str_type, typ],
                 ret_type=typ)
@@ -233,23 +295,28 @@ def typed_dict_pop_callback(ctx: MethodContext) -> Type:
     if (isinstance(ctx.type, TypedDictType)
             and len(ctx.arg_types) >= 1
             and len(ctx.arg_types[0]) == 1):
-        key = try_getting_str_literal(ctx.args[0][0], ctx.arg_types[0][0])
-        if key is None:
+        keys = try_getting_str_literals(ctx.args[0][0], ctx.arg_types[0][0])
+        if keys is None:
             ctx.api.fail(message_registry.TYPEDDICT_KEY_MUST_BE_STRING_LITERAL, ctx.context)
             return AnyType(TypeOfAny.from_error)
 
-        if key in ctx.type.required_keys:
-            ctx.api.msg.typeddict_key_cannot_be_deleted(ctx.type, key, ctx.context)
-        value_type = ctx.type.items.get(key)
-        if value_type:
-            if len(ctx.args[1]) == 0:
-                return value_type
-            elif (len(ctx.arg_types) == 2 and len(ctx.arg_types[1]) == 1
-                  and len(ctx.args[1]) == 1):
-                return UnionType.make_simplified_union([value_type, ctx.arg_types[1][0]])
-        else:
-            ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
-            return AnyType(TypeOfAny.from_error)
+        value_types = []
+        for key in keys:
+            if key in ctx.type.required_keys:
+                ctx.api.msg.typeddict_key_cannot_be_deleted(ctx.type, key, ctx.context)
+
+            value_type = ctx.type.items.get(key)
+            if value_type:
+                value_types.append(value_type)
+            else:
+                ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
+                return AnyType(TypeOfAny.from_error)
+
+        if len(ctx.args[1]) == 0:
+            return make_simplified_union(value_types)
+        elif (len(ctx.arg_types) == 2 and len(ctx.arg_types[1]) == 1
+              and len(ctx.args[1]) == 1):
+            return make_simplified_union([*value_types, ctx.arg_types[1][0]])
     return ctx.default_return_type
 
 
@@ -278,18 +345,35 @@ def typed_dict_setdefault_callback(ctx: MethodContext) -> Type:
     """Type check TypedDict.setdefault and infer a precise return type."""
     if (isinstance(ctx.type, TypedDictType)
             and len(ctx.arg_types) == 2
-            and len(ctx.arg_types[0]) == 1):
-        key = try_getting_str_literal(ctx.args[0][0], ctx.arg_types[0][0])
-        if key is None:
+            and len(ctx.arg_types[0]) == 1
+            and len(ctx.arg_types[1]) == 1):
+        keys = try_getting_str_literals(ctx.args[0][0], ctx.arg_types[0][0])
+        if keys is None:
             ctx.api.fail(message_registry.TYPEDDICT_KEY_MUST_BE_STRING_LITERAL, ctx.context)
             return AnyType(TypeOfAny.from_error)
 
-        value_type = ctx.type.items.get(key)
-        if value_type:
-            return value_type
-        else:
-            ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
-            return AnyType(TypeOfAny.from_error)
+        default_type = ctx.arg_types[1][0]
+
+        value_types = []
+        for key in keys:
+            value_type = ctx.type.items.get(key)
+
+            if value_type is None:
+                ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
+                return AnyType(TypeOfAny.from_error)
+
+            # The signature_callback above can't always infer the right signature
+            # (e.g. when the expression is a variable that happens to be a Literal str)
+            # so we need to handle the check ourselves here and make sure the provided
+            # default can be assigned to all key-value pairs we're updating.
+            if not is_subtype(default_type, value_type):
+                ctx.api.msg.typeddict_setdefault_arguments_inconsistent(
+                    default_type, value_type, ctx.context)
+                return AnyType(TypeOfAny.from_error)
+
+            value_types.append(value_type)
+
+        return make_simplified_union(value_types)
     return ctx.default_return_type
 
 
@@ -304,15 +388,16 @@ def typed_dict_delitem_callback(ctx: MethodContext) -> Type:
     if (isinstance(ctx.type, TypedDictType)
             and len(ctx.arg_types) == 1
             and len(ctx.arg_types[0]) == 1):
-        key = try_getting_str_literal(ctx.args[0][0], ctx.arg_types[0][0])
-        if key is None:
+        keys = try_getting_str_literals(ctx.args[0][0], ctx.arg_types[0][0])
+        if keys is None:
             ctx.api.fail(message_registry.TYPEDDICT_KEY_MUST_BE_STRING_LITERAL, ctx.context)
             return AnyType(TypeOfAny.from_error)
 
-        if key in ctx.type.required_keys:
-            ctx.api.msg.typeddict_key_cannot_be_deleted(ctx.type, key, ctx.context)
-        elif key not in ctx.type.items:
-            ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
+        for key in keys:
+            if key in ctx.type.required_keys:
+                ctx.api.msg.typeddict_key_cannot_be_deleted(ctx.type, key, ctx.context)
+            elif key not in ctx.type.items:
+                ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
     return ctx.default_return_type
 
 
@@ -321,7 +406,7 @@ def typed_dict_update_signature_callback(ctx: MethodSigContext) -> CallableType:
     signature = ctx.default_signature
     if (isinstance(ctx.type, TypedDictType)
             and len(signature.arg_types) == 1):
-        arg_type = signature.arg_types[0]
+        arg_type = get_proper_type(signature.arg_types[0])
         assert isinstance(arg_type, TypedDictType)
         arg_type = arg_type.as_anonymous()
         arg_type = arg_type.copy_modified(required_keys=set())
@@ -331,8 +416,10 @@ def typed_dict_update_signature_callback(ctx: MethodSigContext) -> CallableType:
 
 def int_pow_callback(ctx: MethodContext) -> Type:
     """Infer a more precise return type for int.__pow__."""
-    if (len(ctx.arg_types) == 1
-            and len(ctx.arg_types[0]) == 1):
+    # int.__pow__ has an optional modulo argument,
+    # so we expect 2 argument positions
+    if (len(ctx.arg_types) == 2
+            and len(ctx.arg_types[0]) == 1 and len(ctx.arg_types[1]) == 0):
         arg = ctx.args[0][0]
         if isinstance(arg, IntExpr):
             exponent = arg.value
@@ -346,3 +433,30 @@ def int_pow_callback(ctx: MethodContext) -> Type:
         else:
             return ctx.api.named_generic_type('builtins.float', [])
     return ctx.default_return_type
+
+
+def int_neg_callback(ctx: MethodContext) -> Type:
+    """Infer a more precise return type for int.__neg__.
+
+    This is mainly used to infer the return type as LiteralType
+    if the original underlying object is a LiteralType object
+    """
+    if isinstance(ctx.type, Instance) and ctx.type.last_known_value is not None:
+        value = ctx.type.last_known_value.value
+        fallback = ctx.type.last_known_value.fallback
+        if isinstance(value, int):
+            if is_literal_type_like(ctx.api.type_context[-1]):
+                return LiteralType(value=-value, fallback=fallback)
+            else:
+                return ctx.type.copy_modified(last_known_value=LiteralType(
+                    value=-value,
+                    fallback=ctx.type,
+                    line=ctx.type.line,
+                    column=ctx.type.column,
+                ))
+    elif isinstance(ctx.type, LiteralType):
+        value = ctx.type.value
+        fallback = ctx.type.fallback
+        if isinstance(value, int):
+            return LiteralType(value=-value, fallback=fallback)
+    return ctx.default_return_type
