@@ -44,6 +44,7 @@ from mypy.errors import Errors
 from mypy.semanal_infer import infer_decorator_signature_if_simple
 from mypy.checker import FineGrainedDeferredNode
 from mypy.server.aststrip import SavedAttributes
+from mypy.util import is_typeshed_file
 import mypy.build
 
 if TYPE_CHECKING:
@@ -335,6 +336,12 @@ def semantic_analyze_target(target: str,
         priority = mypy.build.PRI_LOW
         if priority <= state.priorities.get(dep, priority):
             state.priorities[dep] = priority
+
+    # Clear out some stale data to avoid memory leaks and astmerge
+    # validity check confusion
+    analyzer.statement = None
+    del analyzer.cur_mod_node
+
     if analyzer.deferred:
         return [target], analyzer.incomplete, analyzer.progress
     else:
@@ -347,7 +354,7 @@ def check_type_arguments(graph: 'Graph', scc: List[str], errors: Errors) -> None
         assert state.tree
         analyzer = TypeArgumentAnalyzer(errors,
                                         state.options,
-                                        errors.is_typeshed_file(state.path or ''))
+                                        is_typeshed_file(state.path or ''))
         with state.wrap_context():
             with strict_optional_set(state.options.strict_optional):
                 state.tree.accept(analyzer)
@@ -362,7 +369,7 @@ def check_type_arguments_in_targets(targets: List[FineGrainedDeferredNode], stat
     """
     analyzer = TypeArgumentAnalyzer(errors,
                                     state.options,
-                                    errors.is_typeshed_file(state.path or ''))
+                                    is_typeshed_file(state.path or ''))
     with state.wrap_context():
         with strict_optional_set(state.options.strict_optional):
             for target in targets:
