@@ -54,12 +54,18 @@ def isproperty(o: object, attr: str) -> bool:
 
 
 def get_edge_candidates(o: object) -> Iterator[Tuple[object, object]]:
+    # use getattr because mypyc expects dict, not mappingproxy
+    if '__getattribute__' in getattr(type(o), '__dict__'):  # noqa
+        return
     if type(o) not in COLLECTION_TYPE_BLACKLIST:
         for attr in dir(o):
-            if attr not in ATTR_BLACKLIST and hasattr(o, attr) and not isproperty(o, attr):
-                e = getattr(o, attr)
-                if not type(e) in ATOMIC_TYPE_BLACKLIST:
-                    yield attr, e
+            try:
+                if attr not in ATTR_BLACKLIST and hasattr(o, attr) and not isproperty(o, attr):
+                    e = getattr(o, attr)
+                    if not type(e) in ATOMIC_TYPE_BLACKLIST:
+                        yield attr, e
+            except AssertionError:
+                pass
     if isinstance(o, Mapping):
         for k, v in o.items():
             yield k, v
@@ -78,7 +84,7 @@ def get_edges(o: object) -> Iterator[Tuple[object, object]]:
                 yield (s, '__closure__'), e.__closure__  # type: ignore
             if hasattr(e, '__self__'):
                 se = e.__self__  # type: ignore
-                if se is not o and se is not type(o):
+                if se is not o and se is not type(o) and hasattr(s, '__self__'):
                     yield s.__self__, se  # type: ignore
         else:
             if not type(e) in TYPE_BLACKLIST:
