@@ -7,6 +7,7 @@ from mypy.nodes import (
     SliceExpr, CastExpr, RevealExpr, UnaryExpr, ListExpr, TupleExpr, DictExpr, SetExpr,
     IndexExpr, GeneratorExpr, ListComprehension, SetComprehension, DictionaryComprehension,
     ConditionalExpr, TypeApplication, LambdaExpr, StarExpr, BackquoteExpr, AwaitExpr,
+    AssignmentExpr,
 )
 from mypy.traverser import TraverserVisitor
 
@@ -102,6 +103,10 @@ class SubexpressionFinder(TraverserVisitor):
         self.add(e)
         super().visit_reveal_expr(e)
 
+    def visit_assignment_expr(self, e: AssignmentExpr) -> None:
+        self.add(e)
+        super().visit_assignment_expr(e)
+
     def visit_unary_expr(self, e: UnaryExpr) -> None:
         self.add(e)
         super().visit_unary_expr(e)
