@@ -15,7 +15,7 @@ on specified sources.
 import os
 import re
 
-from typing import List, Dict, Any, Tuple, cast
+from typing import List, Dict, Any, Tuple, Union, cast
 
 from mypy import build
 from mypy.modulefinder import BuildSource
@@ -23,7 +23,7 @@ from mypy.errors import CompileError
 from mypy.options import Options
 from mypy.test.config import test_temp_dir
 from mypy.test.data import (
-    DataDrivenTestCase, DataSuite, UpdateFile
+    DataDrivenTestCase, DataSuite, UpdateFile, DeleteFile
 )
 from mypy.test.helpers import (
     assert_string_arrays_equal, parse_options, copy_and_fudge_mtime, assert_module_equivalence,
@@ -35,7 +35,7 @@ from mypy.dmypy_server import Server
 from mypy.config_parser import parse_config_file
 from mypy.find_sources import create_source_list
 
-import pytest  # type: ignore  # no pytest in typeshed
+import pytest
 
 # Set to True to perform (somewhat expensive) checks for duplicate AST nodes after merge
 CHECK_CONSISTENCY = False
@@ -47,16 +47,18 @@ class FineGrainedSuite(DataSuite):
         'fine-grained-cycles.test',
         'fine-grained-blockers.test',
         'fine-grained-modules.test',
+        'fine-grained-follow-imports.test',
         'fine-grained-suggest.test',
     ]
+
     # Whether to use the fine-grained cache in the testing. This is overridden
     # by a trivial subclass to produce a suite that uses the cache.
     use_cache = False
 
-    # Decide whether to skip the test. This could have been structured
-    # as a filter() classmethod also, but we want the tests reported
-    # as skipped, not just elided.
     def should_skip(self, testcase: DataDrivenTestCase) -> bool:
+        # Decide whether to skip the test. This could have been structured
+        # as a filter() classmethod also, but we want the tests reported
+        # as skipped, not just elided.
         if self.use_cache:
             if testcase.only_when == '-only_when_nocache':
                 return True
@@ -108,56 +110,19 @@ class FineGrainedSuite(DataSuite):
 
         for operations in steps:
             step += 1
-            for op in operations:
-                if isinstance(op, UpdateFile):
-                    # Modify/create file
-                    copy_and_fudge_mtime(op.source_path, op.target_path)
-                else:
-                    # Delete file
-                    os.remove(op.path)
-            sources = self.parse_sources(main_src, step, options)
-
-            if step <= num_regular_incremental_steps:
-                new_messages = self.build(build_options, sources)
-            else:
-                new_messages = self.run_check(server, sources)
-
-            updated = []  # type: List[str]
-            changed = []  # type: List[str]
-            targets = []  # type: List[str]
-            if server.fine_grained_manager:
-                if CHECK_CONSISTENCY:
-                    check_consistency(server.fine_grained_manager)
-                all_triggered.append(server.fine_grained_manager.triggered)
-
-                updated = server.fine_grained_manager.updated_modules
-                changed = [mod for mod, file in server.fine_grained_manager.changed_modules]
-                targets = server.fine_grained_manager.processed_targets
-
-            expected_stale = testcase.expected_stale_modules.get(step - 1)
-            if expected_stale is not None:
-                assert_module_equivalence(
-                    'stale' + str(step - 1),
-                    expected_stale, changed)
-
-            expected_rechecked = testcase.expected_rechecked_modules.get(step - 1)
-            if expected_rechecked is not None:
-                assert_module_equivalence(
-                    'rechecked' + str(step - 1),
-                    expected_rechecked, updated)
-
-            expected = testcase.expected_fine_grained_targets.get(step)
-            if expected:
-                assert_target_equivalence(
-                    'targets' + str(step),
-                    expected, targets)
-
-            new_messages = normalize_messages(new_messages)
-
+            output, triggered = self.perform_step(
+                operations,
+                server,
+                options,
+                build_options,
+                testcase,
+                main_src,
+                step,
+                num_regular_incremental_steps,
+            )
             a.append('==')
-            a.extend(new_messages)
-            assert testcase.tmpdir
-            a.extend(self.maybe_suggest(step, server, main_src, testcase.tmpdir.name))
+            a.extend(output)
+            all_triggered.extend(triggered)
 
         # Normalize paths in test output (for Windows).
         a = [line.replace('\\', '/') for line in a]
@@ -177,7 +142,7 @@ class FineGrainedSuite(DataSuite):
     def get_options(self,
                     source: str,
                     testcase: DataDrivenTestCase,
-                    build_cache: bool) -> Options:
+                    build_cache: bool,) -> Options:
         # This handles things like '# flags: --foo'.
         options = parse_options(source, testcase, incremental_step=1)
         options.incremental = True
@@ -188,12 +153,13 @@ class FineGrainedSuite(DataSuite):
         options.use_fine_grained_cache = self.use_cache and not build_cache
         options.cache_fine_grained = self.use_cache
         options.local_partial_types = True
-        if options.follow_imports == 'normal':
+        if re.search('flags:.*--follow-imports', source) is None:
+            # Override the default for follow_imports
             options.follow_imports = 'error'
 
         for name, _ in testcase.files:
             if 'mypy.ini' in name:
-                parse_config_file(options, name)
+                parse_config_file(options, lambda: None, name)
                 break
 
         return options
@@ -231,6 +197,72 @@ class FineGrainedSuite(DataSuite):
             return int(m.group(1))
         return 1
 
+    def perform_step(self,
+                     operations: List[Union[UpdateFile, DeleteFile]],
+                     server: Server,
+                     options: Options,
+                     build_options: Options,
+                     testcase: DataDrivenTestCase,
+                     main_src: str,
+                     step: int,
+                     num_regular_incremental_steps: int) -> Tuple[List[str], List[List[str]]]:
+        """Perform one fine-grained incremental build step (after some file updates/deletions).
+
+        Return (mypy output, triggered targets).
+        """
+        for op in operations:
+            if isinstance(op, UpdateFile):
+                # Modify/create file
+                copy_and_fudge_mtime(op.source_path, op.target_path)
+            else:
+                # Delete file
+                os.remove(op.path)
+        sources = self.parse_sources(main_src, step, options)
+
+        if step <= num_regular_incremental_steps:
+            new_messages = self.build(build_options, sources)
+        else:
+            new_messages = self.run_check(server, sources)
+
+        updated = []  # type: List[str]
+        changed = []  # type: List[str]
+        targets = []  # type: List[str]
+        triggered = []
+        if server.fine_grained_manager:
+            if CHECK_CONSISTENCY:
+                check_consistency(server.fine_grained_manager)
+            triggered.append(server.fine_grained_manager.triggered)
+
+            updated = server.fine_grained_manager.updated_modules
+            changed = [mod for mod, file in server.fine_grained_manager.changed_modules]
+            targets = server.fine_grained_manager.processed_targets
+
+        expected_stale = testcase.expected_stale_modules.get(step - 1)
+        if expected_stale is not None:
+            assert_module_equivalence(
+                'stale' + str(step - 1),
+                expected_stale, changed)
+
+        expected_rechecked = testcase.expected_rechecked_modules.get(step - 1)
+        if expected_rechecked is not None:
+            assert_module_equivalence(
+                'rechecked' + str(step - 1),
+                expected_rechecked, updated)
+
+        expected = testcase.expected_fine_grained_targets.get(step)
+        if expected:
+            assert_target_equivalence(
+                'targets' + str(step),
+                expected, targets)
+
+        new_messages = normalize_messages(new_messages)
+
+        a = new_messages
+        assert testcase.tmpdir
+        a.extend(self.maybe_suggest(step, server, main_src, testcase.tmpdir.name))
+
+        return a, triggered
+
     def parse_sources(self, program_text: str,
                       incremental_step: int,
                       options: Options) -> List[BuildSource]:
