@@ -5,12 +5,13 @@ import os.path
 from typing import Dict, List
 
 from mypy import build
-from mypy.build import BuildSource
+from mypy.modulefinder import BuildSource
+from mypy.defaults import PYTHON3_VERSION
 from mypy.test.helpers import (
-    assert_string_arrays_equal, normalize_error_messages, testfile_pyversion,
+    assert_string_arrays_equal, normalize_error_messages, testfile_pyversion, parse_options
 )
-from mypy.test.data import parse_test_cases, DataDrivenTestCase, DataSuite
-from mypy.test.config import test_data_prefix, test_temp_dir
+from mypy.test.data import DataDrivenTestCase, DataSuite
+from mypy.test.config import test_temp_dir
 from mypy.errors import CompileError
 from mypy.nodes import TypeInfo
 from mypy.options import Options
@@ -29,29 +30,23 @@ semanal_files = ['semanal-basic.test',
                  'semanal-abstractclasses.test',
                  'semanal-namedtuple.test',
                  'semanal-typeddict.test',
+                 'semenal-literal.test',
                  'semanal-classvar.test',
                  'semanal-python2.test']
 
 
-def get_semanal_options() -> Options:
-    options = Options()
+def get_semanal_options(program_text: str, testcase: DataDrivenTestCase) -> Options:
+    options = parse_options(program_text, testcase, 1)
     options.use_builtins_fixtures = True
     options.semantic_analysis_only = True
     options.show_traceback = True
+    options.python_version = PYTHON3_VERSION
     return options
 
 
 class SemAnalSuite(DataSuite):
-    @classmethod
-    def cases(cls) -> List[DataDrivenTestCase]:
-        c = []  # type: List[DataDrivenTestCase]
-        for f in semanal_files:
-            c += parse_test_cases(os.path.join(test_data_prefix, f),
-                                  test_semanal,
-                                  base_path=test_temp_dir,
-                                  optional_out=True,
-                                  native_sep=True)
-        return c
+    files = semanal_files
+    native_sep = True
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         test_semanal(testcase)
@@ -66,7 +61,7 @@ def test_semanal(testcase: DataDrivenTestCase) -> None:
 
     try:
         src = '\n'.join(testcase.input)
-        options = get_semanal_options()
+        options = get_semanal_options(src, testcase)
         options.python_version = testfile_pyversion(testcase.file)
         result = build.build(sources=[BuildSource('main', None, src)],
                              options=options,
@@ -84,15 +79,18 @@ def test_semanal(testcase: DataDrivenTestCase) -> None:
             if (not f.path.endswith((os.sep + 'builtins.pyi',
                                      'typing.pyi',
                                      'mypy_extensions.pyi',
+                                     'typing_extensions.pyi',
                                      'abc.pyi',
-                                     'collections.pyi'))
+                                     'collections.pyi',
+                                     'sys.pyi'))
                     and not os.path.basename(f.path).startswith('_')
                     and not os.path.splitext(
                         os.path.basename(f.path))[0].endswith('_')):
                 a += str(f).split('\n')
     except CompileError as e:
         a = e.messages
-    a = normalize_error_messages(a)
+    if testcase.normalize_output:
+        a = normalize_error_messages(a)
     assert_string_arrays_equal(
         testcase.output, a,
         'Invalid semantic analyzer output ({}, line {})'.format(testcase.file,
@@ -101,19 +99,8 @@ def test_semanal(testcase: DataDrivenTestCase) -> None:
 
 # Semantic analyzer error test cases
 
-# Paths to files containing test case descriptions.
-semanal_error_files = ['semanal-errors.test']
-
-
 class SemAnalErrorSuite(DataSuite):
-    @classmethod
-    def cases(cls) -> List[DataDrivenTestCase]:
-        # Read test cases from test case description files.
-        c = []  # type: List[DataDrivenTestCase]
-        for f in semanal_error_files:
-            c += parse_test_cases(os.path.join(test_data_prefix, f),
-                                  test_semanal_error, test_temp_dir, optional_out=True)
-        return c
+    files = ['semanal-errors.test']
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         test_semanal_error(testcase)
@@ -125,7 +112,7 @@ def test_semanal_error(testcase: DataDrivenTestCase) -> None:
     try:
         src = '\n'.join(testcase.input)
         res = build.build(sources=[BuildSource('main', None, src)],
-                          options=get_semanal_options(),
+                          options=get_semanal_options(src, testcase),
                           alt_lib_path=test_temp_dir)
         a = res.errors
         assert a, 'No errors reported in {}, line {}'.format(testcase.file, testcase.line)
@@ -133,25 +120,18 @@ def test_semanal_error(testcase: DataDrivenTestCase) -> None:
         # Verify that there was a compile error and that the error messages
         # are equivalent.
         a = e.messages
+    if testcase.normalize_output:
+        a = normalize_error_messages(a)
     assert_string_arrays_equal(
-        testcase.output, normalize_error_messages(a),
+        testcase.output, a,
         'Invalid compiler output ({}, line {})'.format(testcase.file, testcase.line))
 
 
 # SymbolNode table export test cases
 
-# Test case descriptions
-semanal_symtable_files = ['semanal-symtable.test']
-
-
 class SemAnalSymtableSuite(DataSuite):
-    @classmethod
-    def cases(cls) -> List[DataDrivenTestCase]:
-        c = []  # type: List[DataDrivenTestCase]
-        for f in semanal_symtable_files:
-            c += parse_test_cases(os.path.join(test_data_prefix, f),
-                                  None, test_temp_dir)
-        return c
+    required_out_section = True
+    files = ['semanal-symtable.test']
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         """Perform a test case."""
@@ -159,7 +139,7 @@ class SemAnalSymtableSuite(DataSuite):
             # Build test case input.
             src = '\n'.join(testcase.input)
             result = build.build(sources=[BuildSource('main', None, src)],
-                                 options=get_semanal_options(),
+                                 options=get_semanal_options(src, testcase),
                                  alt_lib_path=test_temp_dir)
             # The output is the symbol table converted into a string.
             a = result.errors
@@ -179,19 +159,9 @@ class SemAnalSymtableSuite(DataSuite):
 
 
 # Type info export test cases
-
-semanal_typeinfo_files = ['semanal-typeinfo.test']
-
-
 class SemAnalTypeInfoSuite(DataSuite):
-    @classmethod
-    def cases(cls) -> List[DataDrivenTestCase]:
-        """Test case descriptions"""
-        c = []  # type: List[DataDrivenTestCase]
-        for f in semanal_typeinfo_files:
-            c += parse_test_cases(os.path.join(test_data_prefix, f),
-                                  None, test_temp_dir)
-        return c
+    required_out_section = True
+    files = ['semanal-typeinfo.test']
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         """Perform a test case."""
@@ -199,7 +169,7 @@ class SemAnalTypeInfoSuite(DataSuite):
             # Build test case input.
             src = '\n'.join(testcase.input)
             result = build.build(sources=[BuildSource('main', None, src)],
-                                 options=get_semanal_options(),
+                                 options=get_semanal_options(src, testcase),
                                  alt_lib_path=test_temp_dir)
             a = result.errors
             if a:
