@@ -10,14 +10,14 @@ from typing_extensions import Type as TypingType
 import sys
 
 from mypy.types import (
-    TupleType, Instance, FunctionLike, Type, CallableType, TypeVarDef, Overloaded,
+    TupleType, Instance, FunctionLike, Type, CallableType, TypeVarDef, TypeVarLikeDef, Overloaded,
     TypeVarType, UninhabitedType, FormalArgument, UnionType, NoneType, TypedDictType,
     AnyType, TypeOfAny, TypeType, ProperType, LiteralType, get_proper_type, get_proper_types,
     copy_type, TypeAliasType, TypeQuery
 )
 from mypy.nodes import (
     FuncBase, FuncItem, OverloadedFuncDef, TypeInfo, ARG_STAR, ARG_STAR2, ARG_POS,
-    Expression, StrExpr, Var
+    Expression, StrExpr, Var, Decorator, SYMBOL_FUNCBASE_TYPES
 )
 from mypy.maptype import map_instance_to_supertype
 from mypy.expandtype import expand_type_by_instance, expand_type
@@ -113,7 +113,7 @@ def class_callable(init_type: CallableType, info: TypeInfo, type_type: Instance,
                    special_sig: Optional[str],
                    is_new: bool, orig_self_type: Optional[Type] = None) -> CallableType:
     """Create a type object type based on the signature of __init__."""
-    variables = []  # type: List[TypeVarDef]
+    variables = []  # type: List[TypeVarLikeDef]
     variables.extend(info.defn.type_vars)
     variables.extend(init_type.variables)
 
@@ -227,13 +227,15 @@ def bind_self(method: F, original_type: Optional[Type] = None, is_classmethod: b
         # TODO: infer bounds on the type of *args?
         return cast(F, func)
     self_param_type = get_proper_type(func.arg_types[0])
+
+    variables = []  # type: Sequence[TypeVarLikeDef]
     if func.variables and supported_self_type(self_param_type):
         if original_type is None:
             # TODO: type check method override (see #7861).
             original_type = erase_to_bound(self_param_type)
         original_type = get_proper_type(original_type)
 
-        all_ids = [x.id for x in func.variables]
+        all_ids = func.type_var_ids()
         typeargs = infer_type_arguments(all_ids, self_param_type, original_type,
                                         is_supertype=True)
         if (is_classmethod
@@ -344,26 +346,53 @@ def make_simplified_union(items: Sequence[Type],
     from mypy.subtypes import is_proper_subtype
 
     removed = set()  # type: Set[int]
-    for i, ti in enumerate(items):
-        if i in removed: continue
-        # Keep track of the truishness info for deleted subtypes which can be relevant
-        cbt = cbf = False
-        for j, tj in enumerate(items):
-            if i != j and is_proper_subtype(tj, ti, keep_erased_types=keep_erased):
-                # We found a redundant item in the union.
-                removed.add(j)
-                cbt = cbt or tj.can_be_true
-                cbf = cbf or tj.can_be_false
-        # if deleted subtypes had more general truthiness, use that
-        if not ti.can_be_true and cbt:
-            items[i] = true_or_false(ti)
-        elif not ti.can_be_false and cbf:
-            items[i] = true_or_false(ti)
+
+    # Avoid slow nested for loop for Union of Literal of strings (issue #9169)
+    if all((isinstance(item, LiteralType) and
+            item.fallback.type.fullname == 'builtins.str')
+           for item in items):
+        seen = set()    # type: Set[str]
+        for index, item in enumerate(items):
+            assert isinstance(item, LiteralType)
+            assert isinstance(item.value, str)
+            if item.value in seen:
+                removed.add(index)
+            seen.add(item.value)
+
+    else:
+        for i, ti in enumerate(items):
+            if i in removed: continue
+            # Keep track of the truishness info for deleted subtypes which can be relevant
+            cbt = cbf = False
+            for j, tj in enumerate(items):
+                if i != j and is_proper_subtype(tj, ti, keep_erased_types=keep_erased):
+                    # We found a redundant item in the union.
+                    removed.add(j)
+                    cbt = cbt or tj.can_be_true
+                    cbf = cbf or tj.can_be_false
+            # if deleted subtypes had more general truthiness, use that
+            if not ti.can_be_true and cbt:
+                items[i] = true_or_false(ti)
+            elif not ti.can_be_false and cbf:
+                items[i] = true_or_false(ti)
 
     simplified_set = [items[i] for i in range(len(items)) if i not in removed]
     return UnionType.make_union(simplified_set, line, column)
 
 
+def get_type_special_method_bool_ret_type(t: Type) -> Optional[Type]:
+    t = get_proper_type(t)
+
+    if isinstance(t, Instance):
+        bool_method = t.type.names.get("__bool__", None)
+        if bool_method:
+            callee = get_proper_type(bool_method.type)
+            if isinstance(callee, CallableType):
+                return callee.ret_type
+
+    return None
+
+
 def true_only(t: Type) -> ProperType:
     """
     Restricted version of t with only True-ish values
@@ -379,8 +408,16 @@ def true_only(t: Type) -> ProperType:
     elif isinstance(t, UnionType):
         # The true version of a union type is the union of the true versions of its components
         new_items = [true_only(item) for item in t.items]
-        return make_simplified_union(new_items, line=t.line, column=t.column)
+        can_be_true_items = [item for item in new_items if item.can_be_true]
+        return make_simplified_union(can_be_true_items, line=t.line, column=t.column)
     else:
+        ret_type = get_type_special_method_bool_ret_type(t)
+
+        if ret_type and ret_type.can_be_false and not ret_type.can_be_true:
+            new_t = copy_type(t)
+            new_t.can_be_true = False
+            return new_t
+
         new_t = copy_type(t)
         new_t.can_be_false = False
         return new_t
@@ -406,8 +443,16 @@ def false_only(t: Type) -> ProperType:
     elif isinstance(t, UnionType):
         # The false version of a union type is the union of the false versions of its components
         new_items = [false_only(item) for item in t.items]
-        return make_simplified_union(new_items, line=t.line, column=t.column)
+        can_be_false_items = [item for item in new_items if item.can_be_false]
+        return make_simplified_union(can_be_false_items, line=t.line, column=t.column)
     else:
+        ret_type = get_type_special_method_bool_ret_type(t)
+
+        if ret_type and ret_type.can_be_true and not ret_type.can_be_false:
+            new_t = copy_type(t)
+            new_t.can_be_false = False
+            return new_t
+
         new_t = copy_type(t)
         new_t.can_be_true = False
         return new_t
@@ -429,7 +474,9 @@ def true_or_false(t: Type) -> ProperType:
     return new_t
 
 
-def erase_def_to_union_or_bound(tdef: TypeVarDef) -> Type:
+def erase_def_to_union_or_bound(tdef: TypeVarLikeDef) -> Type:
+    # TODO(shantanu): fix for ParamSpecDef
+    assert isinstance(tdef, TypeVarDef)
     if tdef.values:
         return make_simplified_union(tdef.values)
     else:
@@ -564,6 +611,24 @@ def try_getting_literals_from_type(typ: Type,
     return literals
 
 
+def is_literal_type_like(t: Optional[Type]) -> bool:
+    """Returns 'true' if the given type context is potentially either a LiteralType,
+    a Union of LiteralType, or something similar.
+    """
+    t = get_proper_type(t)
+    if t is None:
+        return False
+    elif isinstance(t, LiteralType):
+        return True
+    elif isinstance(t, UnionType):
+        return any(is_literal_type_like(item) for item in t.items)
+    elif isinstance(t, TypeVarType):
+        return (is_literal_type_like(t.upper_bound)
+                or any(is_literal_type_like(item) for item in t.values))
+    else:
+        return False
+
+
 def get_enum_values(typ: Instance) -> List[str]:
     """Return the list of values for an Enum."""
     return [name for name, sym in typ.type.names.items() if isinstance(sym.node, Var)]
@@ -586,10 +651,12 @@ def is_singleton_type(typ: Type) -> bool:
     constructing two distinct instances of 100001.
     """
     typ = get_proper_type(typ)
-    # TODO: Also make this return True if the type is a bool LiteralType.
+    # TODO:
     # Also make this return True if the type corresponds to ... (ellipsis) or NotImplemented?
     return (
-            isinstance(typ, NoneType) or (isinstance(typ, LiteralType) and typ.is_enum_literal())
+            isinstance(typ, NoneType)
+            or (isinstance(typ, LiteralType)
+                and (typ.is_enum_literal() or isinstance(typ.value, bool)))
             or (isinstance(typ, Instance) and typ.type.is_enum and len(get_enum_values(typ)) == 1)
     )
 
@@ -623,6 +690,9 @@ def try_expanding_enum_to_union(typ: Type, target_fullname: str) -> ProperType:
         for name, symbol in typ.type.names.items():
             if not isinstance(symbol.node, Var):
                 continue
+            # Skip "_order_" and "__order__", since Enum will remove it
+            if name in ("_order_", "__order__"):
+                continue
             new_items.append(LiteralType(name, typ))
         # SymbolTables are really just dicts, and dicts are guaranteed to preserve
         # insertion order only starting with Python 3.7. So, we sort these for older
@@ -637,10 +707,11 @@ def try_expanding_enum_to_union(typ: Type, target_fullname: str) -> ProperType:
         return typ
 
 
-def coerce_to_literal(typ: Type) -> ProperType:
+def coerce_to_literal(typ: Type) -> Type:
     """Recursively converts any Instances that have a last_known_value or are
     instances of enum types with a single value into the corresponding LiteralType.
     """
+    original_type = typ
     typ = get_proper_type(typ)
     if isinstance(typ, UnionType):
         new_items = [coerce_to_literal(item) for item in typ.items]
@@ -652,7 +723,7 @@ def coerce_to_literal(typ: Type) -> ProperType:
             enum_values = get_enum_values(typ)
             if len(enum_values) == 1:
                 return LiteralType(value=enum_values[0], fallback=typ)
-    return typ
+    return original_type
 
 
 def get_type_vars(tp: Type) -> List[TypeVarType]:
@@ -671,3 +742,31 @@ class TypeVarExtractor(TypeQuery[List[TypeVarType]]):
 
     def visit_type_var(self, t: TypeVarType) -> List[TypeVarType]:
         return [t]
+
+
+def custom_special_method(typ: Type, name: str, check_all: bool = False) -> bool:
+    """Does this type have a custom special method such as __format__() or __eq__()?
+
+    If check_all is True ensure all items of a union have a custom method, not just some.
+    """
+    typ = get_proper_type(typ)
+    if isinstance(typ, Instance):
+        method = typ.type.get(name)
+        if method and isinstance(method.node, (SYMBOL_FUNCBASE_TYPES, Decorator, Var)):
+            if method.node.info:
+                return not method.node.info.fullname.startswith('builtins.')
+        return False
+    if isinstance(typ, UnionType):
+        if check_all:
+            return all(custom_special_method(t, name, check_all) for t in typ.items)
+        return any(custom_special_method(t, name) for t in typ.items)
+    if isinstance(typ, TupleType):
+        return custom_special_method(tuple_fallback(typ), name, check_all)
+    if isinstance(typ, CallableType) and typ.is_type_obj():
+        # Look up __method__ on the metaclass for class objects.
+        return custom_special_method(typ.fallback, name, check_all)
+    if isinstance(typ, AnyType):
+        # Avoid false positives in uncertain cases.
+        return True
+    # TODO: support other types (see ExpressionChecker.has_member())?
+    return False
