@@ -37,10 +37,10 @@ from mypy.util import write_junit_xml
 from mypyc.namegen import exported_name
 from mypyc.options import CompilerOptions
 from mypyc.errors import Errors
-from mypyc.common import shared_lib_name
-from mypyc.ops import format_modules
+from mypyc.common import RUNTIME_C_FILES, shared_lib_name
+from mypyc.ir.pprint import format_modules
 
-from mypyc import emitmodule
+from mypyc.codegen import emitmodule
 
 if TYPE_CHECKING:
     from distutils.core import Extension  # noqa
@@ -536,7 +536,7 @@ def mypycify(
     # compiler invocations.
     shared_cfilenames = []
     if not compiler_options.include_runtime_files:
-        for name in ['CPy.c', 'getargs.c']:
+        for name in RUNTIME_C_FILES:
             rt_file = os.path.join(build_dir, name)
             with open(os.path.join(include_dir(), name), encoding='utf-8') as f:
                 write_file(rt_file, f.read())
