@@ -21,11 +21,11 @@ RTypes.
 """
 
 from abc import abstractmethod
-from typing import Optional, Union, List, Dict, Generic, TypeVar
+from typing import Optional, Union, List, Dict, Generic, TypeVar, Tuple
 
 from typing_extensions import Final, ClassVar, TYPE_CHECKING
 
-from mypyc.common import JsonDict, short_name
+from mypyc.common import JsonDict, short_name, IS_32_BIT_PLATFORM, PLATFORM_SIZE
 from mypyc.namegen import NameGenerator
 
 if TYPE_CHECKING:
@@ -119,6 +119,10 @@ class RTypeVisitor(Generic[T]):
     def visit_rtuple(self, typ: 'RTuple') -> T:
         raise NotImplementedError
 
+    @abstractmethod
+    def visit_rstruct(self, typ: 'RStruct') -> T:
+        raise NotImplementedError
+
     @abstractmethod
     def visit_rvoid(self, typ: 'RVoid') -> T:
         raise NotImplementedError
@@ -128,7 +132,7 @@ class RVoid(RType):
     """The void type (no value).
 
     This is a singleton -- use void_rtype (below) to refer to this instead of
-    constructing a new instace.
+    constructing a new instance.
     """
 
     is_unboxed = False
@@ -167,20 +171,28 @@ class RPrimitive(RType):
                  name: str,
                  is_unboxed: bool,
                  is_refcounted: bool,
-                 ctype: str = 'PyObject *') -> None:
+                 ctype: str = 'PyObject *',
+                 size: int = PLATFORM_SIZE) -> None:
         RPrimitive.primitive_map[name] = self
 
         self.name = name
         self.is_unboxed = is_unboxed
         self._ctype = ctype
         self.is_refcounted = is_refcounted
+        self.size = size
+        # TODO: For low-level integers, they actually don't have undefined values
+        #       we need to figure out some way to represent here.
         if ctype == 'CPyTagged':
             self.c_undefined = 'CPY_INT_TAG'
+        elif ctype in ('int32_t', 'int64_t', 'CPyPtr'):
+            self.c_undefined = '0'
         elif ctype == 'PyObject *':
             # Boxed types use the null pointer as the error value.
             self.c_undefined = 'NULL'
         elif ctype == 'char':
             self.c_undefined = '2'
+        elif ctype == 'PyObject **':
+            self.c_undefined = 'NULL'
         else:
             assert False, 'Unrecognized ctype: %r' % ctype
 
@@ -213,6 +225,10 @@ class RPrimitive(RType):
 object_rprimitive = RPrimitive('builtins.object', is_unboxed=False,
                                is_refcounted=True)  # type: Final
 
+# represents a low level pointer of an object
+object_pointer_rprimitive = RPrimitive('object_ptr', is_unboxed=False,
+                               is_refcounted=False, ctype='PyObject **')  # type: Final
+
 # Arbitrary-precision integer (corresponds to Python 'int'). Small
 # enough values are stored unboxed, while large integers are
 # represented as a tagged pointer to a Python 'int' PyObject. The
@@ -234,19 +250,42 @@ int_rprimitive = RPrimitive('builtins.int', is_unboxed=True, is_refcounted=True,
 short_int_rprimitive = RPrimitive('short_int', is_unboxed=True, is_refcounted=False,
                                   ctype='CPyTagged')  # type: Final
 
+# low level integer (corresponds to C's 'int's).
+int32_rprimitive = RPrimitive('int32', is_unboxed=True, is_refcounted=False,
+                              ctype='int32_t', size=4)  # type: Final
+int64_rprimitive = RPrimitive('int64', is_unboxed=True, is_refcounted=False,
+                              ctype='int64_t', size=8)  # type: Final
+# integer alias
+c_int_rprimitive = int32_rprimitive
+if IS_32_BIT_PLATFORM:
+    c_pyssize_t_rprimitive = int32_rprimitive
+else:
+    c_pyssize_t_rprimitive = int64_rprimitive
+
+# low level pointer, represented as integer in C backends
+pointer_rprimitive = RPrimitive('ptr', is_unboxed=True, is_refcounted=False,
+                              ctype='CPyPtr')  # type: Final
+
 # Floats are represent as 'float' PyObject * values. (In the future
 # we'll likely switch to a more efficient, unboxed representation.)
 float_rprimitive = RPrimitive('builtins.float', is_unboxed=False,
                               is_refcounted=True)  # type: Final
 
-# An unboxed boolean value. This actually has three possible values
-# (0 -> False, 1 -> True, 2 -> error).
+# An unboxed Python bool value. This actually has three possible values
+# (0 -> False, 1 -> True, 2 -> error). If you only need True/False, use
+# bit_rprimitive instead.
 bool_rprimitive = RPrimitive('builtins.bool', is_unboxed=True, is_refcounted=False,
-                             ctype='char')  # type: Final
+                             ctype='char', size=1)  # type: Final
+
+# A low-level boolean value with two possible values: 0 and 1. Any
+# other value results in undefined behavior. Undefined or error values
+# are not supported.
+bit_rprimitive = RPrimitive('bit', is_unboxed=True, is_refcounted=False,
+                            ctype='char', size=1)  # type: Final
 
 # The 'None' value. The possible values are 0 -> None and 2 -> error.
 none_rprimitive = RPrimitive('builtins.None', is_unboxed=True, is_refcounted=False,
-                             ctype='char')  # type: Final
+                             ctype='char', size=1)  # type: Final
 
 # Python list object (or an instance of a subclass of list).
 list_rprimitive = RPrimitive('builtins.list', is_unboxed=False, is_refcounted=True)  # type: Final
@@ -267,6 +306,10 @@ tuple_rprimitive = RPrimitive('builtins.tuple', is_unboxed=False,
                               is_refcounted=True)  # type: Final
 
 
+def is_tagged(rtype: RType) -> bool:
+    return rtype is int_rprimitive or rtype is short_int_rprimitive
+
+
 def is_int_rprimitive(rtype: RType) -> bool:
     return rtype is int_rprimitive
 
@@ -275,6 +318,22 @@ def is_short_int_rprimitive(rtype: RType) -> bool:
     return rtype is short_int_rprimitive
 
 
+def is_int32_rprimitive(rtype: RType) -> bool:
+    return rtype is int32_rprimitive
+
+
+def is_int64_rprimitive(rtype: RType) -> bool:
+    return rtype is int64_rprimitive
+
+
+def is_c_py_ssize_t_rprimitive(rtype: RType) -> bool:
+    return rtype is c_pyssize_t_rprimitive
+
+
+def is_pointer_rprimitive(rtype: RType) -> bool:
+    return rtype is pointer_rprimitive
+
+
 def is_float_rprimitive(rtype: RType) -> bool:
     return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.float'
 
@@ -283,6 +342,10 @@ def is_bool_rprimitive(rtype: RType) -> bool:
     return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.bool'
 
 
+def is_bit_rprimitive(rtype: RType) -> bool:
+    return isinstance(rtype, RPrimitive) and rtype.name == 'bit'
+
+
 def is_object_rprimitive(rtype: RType) -> bool:
     return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.object'
 
@@ -338,6 +401,10 @@ class TupleNameVisitor(RTypeVisitor[str]):
         parts = [elem.accept(self) for elem in t.types]
         return 'T{}{}'.format(len(parts), ''.join(parts))
 
+    def visit_rstruct(self, t: 'RStruct') -> str:
+        assert False
+        return ""
+
     def visit_rvoid(self, t: 'RVoid') -> str:
         assert False, "rvoid in tuple?"
 
@@ -410,6 +477,113 @@ dict_next_rtuple_single = RTuple(
 )
 
 
+def compute_rtype_alignment(typ: RType) -> int:
+    """Compute alignment of a given type based on platform alignment rule"""
+    platform_alignment = PLATFORM_SIZE
+    if isinstance(typ, RPrimitive):
+        return typ.size
+    elif isinstance(typ, RInstance):
+        return platform_alignment
+    elif isinstance(typ, RUnion):
+        return platform_alignment
+    else:
+        if isinstance(typ, RTuple):
+            items = list(typ.types)
+        elif isinstance(typ, RStruct):
+            items = typ.types
+        else:
+            assert False, "invalid rtype for computing alignment"
+        max_alignment = max([compute_rtype_alignment(item) for item in items])
+        return max_alignment
+
+
+def compute_rtype_size(typ: RType) -> int:
+    """Compute unaligned size of rtype"""
+    if isinstance(typ, RPrimitive):
+        return typ.size
+    elif isinstance(typ, RTuple):
+        return compute_aligned_offsets_and_size(list(typ.types))[1]
+    elif isinstance(typ, RUnion):
+        return PLATFORM_SIZE
+    elif isinstance(typ, RStruct):
+        return compute_aligned_offsets_and_size(typ.types)[1]
+    elif isinstance(typ, RInstance):
+        return PLATFORM_SIZE
+    else:
+        assert False, "invalid rtype for computing size"
+
+
+def compute_aligned_offsets_and_size(types: List[RType]) -> Tuple[List[int], int]:
+    """Compute offsets and total size of a list of types after alignment
+
+    Note that the types argument are types of values that are stored
+    sequentially with platform default alignment.
+    """
+    unaligned_sizes = [compute_rtype_size(typ) for typ in types]
+    alignments = [compute_rtype_alignment(typ) for typ in types]
+
+    current_offset = 0
+    offsets = []
+    final_size = 0
+    for i in range(len(unaligned_sizes)):
+        offsets.append(current_offset)
+        if i + 1 < len(unaligned_sizes):
+            cur_size = unaligned_sizes[i]
+            current_offset += cur_size
+            next_alignment = alignments[i + 1]
+            # compute aligned offset,
+            # check https://en.wikipedia.org/wiki/Data_structure_alignment for more information
+            current_offset = (current_offset + (next_alignment - 1)) & -next_alignment
+        else:
+            struct_alignment = max(alignments)
+            final_size = current_offset + unaligned_sizes[i]
+            final_size = (final_size + (struct_alignment - 1)) & -struct_alignment
+    return offsets, final_size
+
+
+class RStruct(RType):
+    """Represent CPython structs"""
+    def __init__(self,
+                 name: str,
+                 names: List[str],
+                 types: List[RType]) -> None:
+        self.name = name
+        self.names = names
+        self.types = types
+        # generate dummy names
+        if len(self.names) < len(self.types):
+            for i in range(len(self.types) - len(self.names)):
+                self.names.append('_item' + str(i))
+        self.offsets, self.size = compute_aligned_offsets_and_size(types)
+        self._ctype = name
+
+    def accept(self, visitor: 'RTypeVisitor[T]') -> T:
+        return visitor.visit_rstruct(self)
+
+    def __str__(self) -> str:
+        # if not tuple(unamed structs)
+        return '%s{%s}' % (self.name, ', '.join(name + ":" + str(typ)
+                                                for name, typ in zip(self.names, self.types)))
+
+    def __repr__(self) -> str:
+        return '<RStruct %s{%s}>' % (self.name, ', '.join(name + ":" + repr(typ) for name, typ
+                                                          in zip(self.names, self.types)))
+
+    def __eq__(self, other: object) -> bool:
+        return (isinstance(other, RStruct) and self.name == other.name
+                and self.names == other.names and self.types == other.types)
+
+    def __hash__(self) -> int:
+        return hash((self.name, tuple(self.names), tuple(self.types)))
+
+    def serialize(self) -> JsonDict:
+        assert False
+
+    @classmethod
+    def deserialize(cls, data: JsonDict, ctx: 'DeserMaps') -> 'RStruct':
+        assert False
+
+
 class RInstance(RType):
     """Instance of user-defined class (compiled to C extension class).
 
@@ -513,3 +687,38 @@ def optional_value_type(rtype: RType) -> Optional[RType]:
 def is_optional_type(rtype: RType) -> bool:
     """Is rtype an optional type with exactly two union items?"""
     return optional_value_type(rtype) is not None
+
+
+PyObject = RStruct(
+    name='PyObject',
+    names=['ob_refcnt', 'ob_type'],
+    types=[c_pyssize_t_rprimitive, pointer_rprimitive])
+
+PyVarObject = RStruct(
+    name='PyVarObject',
+    names=['ob_base', 'ob_size'],
+    types=[PyObject, c_pyssize_t_rprimitive])
+
+setentry = RStruct(
+    name='setentry',
+    names=['key', 'hash'],
+    types=[pointer_rprimitive, c_pyssize_t_rprimitive])
+
+smalltable = RStruct(
+    name='smalltable',
+    names=[],
+    types=[setentry] * 8)
+
+PySetObject = RStruct(
+    name='PySetObject',
+    names=['ob_base', 'fill', 'used', 'mask', 'table', 'hash', 'finger',
+           'smalltable', 'weakreflist'],
+    types=[PyObject, c_pyssize_t_rprimitive, c_pyssize_t_rprimitive, c_pyssize_t_rprimitive,
+           pointer_rprimitive, c_pyssize_t_rprimitive, c_pyssize_t_rprimitive, smalltable,
+           pointer_rprimitive])
+
+PyListObject = RStruct(
+    name='PyListObject',
+    names=['ob_base', 'ob_item', 'allocated'],
+    types=[PyObject, pointer_rprimitive, c_pyssize_t_rprimitive]
+)
