@@ -13,7 +13,7 @@ class MethodSigPlugin(Plugin):
 
 def _str_to_int(api: CheckerPluginInterface, typ: Type) -> Type:
     if isinstance(typ, Instance):
-        if typ.type.fullname() == 'builtins.str':
+        if typ.type.fullname == 'builtins.str':
             return api.named_generic_type('builtins.int', [])
         elif typ.args:
             return typ.copy_modified(args=[_str_to_int(api, t) for t in typ.args])
